@@ -16,13 +16,8 @@
 # GPL Exception version 1.1, which can be found in the file
 # GPL_EXCEPTION.txt in this package.
 # 
-# Please review the following information to ensure GNU General
-# Public Licensing requirements will be met:
-# http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-# you are unsure which license is appropriate for your use, please
-# review the following information:
-# http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-# or contact the sales department at sales@riverbankcomputing.com.
+# If you are unsure which license is appropriate for your use, please
+# contact the sales department at sales@riverbankcomputing.com.
 # 
 # This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 # WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040803
-pyqt_version_str = "4.8.3"
+pyqt_version = 0x040805
+pyqt_version_str = "4.8.5"
 
-sip_min_version = 0x040c01
+sip_min_version = 0x040c02
 
 qt_version = 0
 qt_edition = ""
@@ -175,14 +170,9 @@ def create_optparser():
     g.add_option("--assume-shared", action="store_true", default=False,
             dest="assume_shared", help="assume that the Qt libraries have "
             "been built as shared libraries [default: check]")
-
-    # This test can be removed when there is a hard dependency on a later
-    # version.
-    if sipcfg.sip_version >= 0x040c00:
-        g.add_option("-T", "--no-timestamp", action="store_true",
-                default=False, dest="no_timestamp", help="suppress timestamps "
-                "in the header comments of generated code [default: include "
-                "timestamps]")
+    g.add_option("-T", "--no-timestamp", action="store_true", default=False,
+            dest="no_timestamp", help="suppress timestamps in the header "
+            "comments of generated code [default: include timestamps]")
 
     if sys.platform != 'win32':
         if sys.platform in ('linux2', 'darwin'):
@@ -343,7 +333,8 @@ class ConfigurePyQt4:
             0x040603: "Qt_4_6_2",
             0x040700: "Qt_4_6_3",
             0x040701: "Qt_4_7_0",
-            0x050000: "Qt_4_7_1"
+            0x040702: "Qt_4_7_1",
+            0x050000: "Qt_4_7_2"
         }
 
     def check_modules(self):
@@ -472,7 +463,18 @@ class ConfigurePyQt4:
 
         if "QtOpenGL" in pyqt_modules:
             generate_OpenGL_extras()
-            generate_code("QtOpenGL")
+
+            qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtOpenGL", "qpyopengl")
+
+            if opts.bigqt:
+                cons_xtra_incdirs.append(qpy_inc_dir)
+                cons_xtra_libdirs.append(qpy_lib_dir)
+                cons_xtra_libs.append(qpy_lib)
+
+                generate_code("QtOpenGL")
+            else:
+                generate_code("QtOpenGL", extra_include_dirs=[qpy_inc_dir],
+                        extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
 
         if "QtScript" in pyqt_modules:
             generate_code("QtScript")
@@ -748,6 +750,9 @@ class ConfigurePyQt4:
         if "QtDesigner" in pyqt_modules:
             qpylibs["QtDesigner"] = "qpydesigner.pro"
 
+        if "QtOpenGL" in pyqt_modules:
+            qpylibs["QtOpenGL"] = "qpyopengl.pro"
+
         # Run qmake to generate the Makefiles.
         qmake_args = fix_qmake_args()
         cwd = os.getcwd()
@@ -773,7 +778,7 @@ class ConfigurePyQt4:
                 f.write("QMAKE_MACOSX_DEPLOYMENT_TARGET = %s\n" % sipcfg.deployment_target)
 
             inc_path = [sipcfg.py_inc_dir]
-            if qpy in ("QtCore", "QtDeclarative"):
+            if qpy in ("QtCore", "QtDeclarative", "QtOpenGL"):
                 if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
                     inc_path.insert(0, sipcfg.sip_inc_dir)
 
@@ -940,15 +945,21 @@ include(%s)
                     # We need to work out how to specify the right framework
                     # version.
                     link = "-framework Python"
-                elif ("--enable-shared" in ducfg.get("CONFIG_ARGS", "") and
-                      glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor))):
-                    lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
+                elif "--enable-shared" in ducfg.get("CONFIG_ARGS", ""):
+                    if glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor)):
+                        lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
+                    elif glob.glob("%s/libpython%d.%d*" % (ducfg["LIBDIR"], py_major, py_minor)):
+                        lib_dir_flag = quote("-L%s" % ducfg["LIBDIR"])
+                    else:
+                        sipconfig.inform("Qt Designer plugin disabled because Python library couldn't be found")
+                        opts.designer_plugin = False
+
                     link = "%s -lpython%d.%d%s" % (lib_dir_flag, py_major, py_minor, abi)
                 else:
                     sipconfig.inform("Qt Designer plugin disabled because Python library is static")
                     opts.designer_plugin = False
 
-                pysh_lib = ducfg["LDLIBRARY"]
+                pysh_lib = ducfg.get("LDLIBRARY", "")
 
             if opts.designer_plugin:
                 sipconfig.inform("Creating Qt Designer plugin Makefile...")
@@ -1913,8 +1924,12 @@ def get_qt_configuration():
     qmake_args = fix_qmake_args("-o " + make_file)
 
     if sys.platform == 'win32':
-        exe_file = os.path.join("release", app + ".exe")
-        make_target = " release"
+        if opts.debug:
+            exe_file = os.path.join("debug", app + ".exe")
+            make_target = " debug"
+        else:
+            exe_file = os.path.join("release", app + ".exe")
+            make_target = " release"
     elif sys.platform == "darwin":
         exe_file = os.path.join(app + ".app", "Contents", "MacOS", app)
     else:
@@ -2149,10 +2164,6 @@ def main():
     p = create_optparser()
     opts, args = p.parse_args()
 
-    # This can be removed when there is a hard dependency on a later version.
-    if sipcfg.sip_version < 0x040c00:
-        opts.no_timestamp = False
-
     # Provide defaults for platform-specific options.
     if sys.platform == 'win32':
         opts.qmake = find_default_qmake()
