@@ -2,24 +2,27 @@
  * This is the Qt Designer plugin that collects all the Python plugins it can
  * find as a widget collection to Designer.
  *
- * Copyright (c) 2007
- * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of PyQt.
  * 
- * This copy of PyQt is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published by
- * the Free Software Foundation and appearing in the file LICENSE included in the
- * packaging of this file.
+ * This file may be used under the terms of the GNU General Public
+ * License versions 2.0 or 3.0 as published by the Free Software
+ * Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+ * included in the packaging of this file.  Alternatively you may (at
+ * your option) use any later version of the GNU General Public
+ * License if such license has been publicly approved by Riverbank
+ * Computing Limited (or its successors, if any) and the KDE Free Qt
+ * Foundation. In addition, as a special exception, Riverbank gives you
+ * certain additional rights. These rights are described in the Riverbank
+ * GPL Exception version 1.1, which can be found in the file
+ * GPL_EXCEPTION.txt in this package.
  * 
- * PyQt is supplied in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
- * details.
+ * If you are unsure which license is appropriate for your use, please
+ * contact the sales department at sales@riverbankcomputing.com.
  * 
- * You should have received a copy of the GNU General Public License along with
- * PyQt; see the file LICENSE.  If not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+ * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
@@ -35,6 +38,7 @@
 #include <QLibrary>
 #include <QLibraryInfo>
 #include <QStringList>
+#include <QVector>
 
 #include "../qpy/QtDesigner/qpydesignercustomwidgetplugin.h"
 
@@ -172,6 +176,50 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
         dir = QDir::convertSeparators(dir);
 #endif
 
+#if PY_MAJOR_VERSION >= 3
+        // This is a copy of qpycore_PyObject_FromQString().
+
+#if defined(Py_UNICODE_WIDE)
+#if QT_VERSION >= 0x040200
+        QVector<uint> ucs4 = dir.toUcs4();
+
+        PyObject *dobj = PyUnicode_FromUnicode(NULL, ucs4.size());
+
+        if (!dobj)
+        {
+            PyErr_Print();
+            continue;
+        }
+
+        memcpy(PyUnicode_AS_UNICODE(dobj), ucs4.constData(),
+                ucs4.size() * sizeof (Py_UNICODE));
+#else
+        PyObject *dobj = PyUnicode_FromUnicode(0, dir.length());
+
+        if (!dobj)
+        {
+            PyErr_Print();
+            continue;
+        }
+
+        Py_UNICODE *pyu = PyUnicode_AS_UNICODE(dobj);
+
+        for (int i = 0; i < dir.length(); ++i)
+            *pyu++ = (dir.at(i)).unicode();
+#endif
+#else
+        PyObject *dobj = PyUnicode_FromUnicode(0, dir.length());
+
+        if (!dobj)
+        {
+            PyErr_Print();
+            continue;
+        }
+
+        memcpy(PyUnicode_AS_UNICODE(dobj), dir.utf16(),
+                dir.length() * sizeof (Py_UNICODE));
+#endif
+#else
         PyObject *dobj = PyString_FromString(dir.toAscii().constData());
 
         if (!dobj)
@@ -179,6 +227,7 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
             PyErr_Print();
             continue;
         }
+#endif
 
         // Add the directory to sys.path.
         int rc = PyList_Append(sys_path, dobj);
@@ -265,9 +314,13 @@ QList<QDesignerCustomWidgetInterface *> PyCustomWidgets::customWidgets() const
 
 
 // Return the named attribute object from the named module.
-PyObject *PyCustomWidgets::getModuleAttr(char *module, char *attr)
+PyObject *PyCustomWidgets::getModuleAttr(const char *module, const char *attr)
 {
+#if PY_VERSION_HEX >= 0x02050000
     PyObject *mod = PyImport_ImportModule(module);
+#else
+    PyObject *mod = PyImport_ImportModule(const_cast<char *>(module));
+#endif
 
     if (!mod)
     {
@@ -275,7 +328,12 @@ PyObject *PyCustomWidgets::getModuleAttr(char *module, char *attr)
         return 0;
     }
 
+#if PY_VERSION_HEX >= 0x02050000
     PyObject *obj = PyObject_GetAttrString(mod, attr);
+#else
+    PyObject *obj = PyObject_GetAttrString(mod, const_cast<char *>(attr));
+#endif
+
     Py_DECREF(mod);
 
     if (!obj)
