@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -1428,8 +1423,16 @@ PyObject *Chimera::toPyObject(void *cpp) const
         break;
 
     case QMetaType::UInt:
-        py = SIPLong_FromLong(*reinterpret_cast<unsigned int *>(cpp));
-        break;
+        {
+            long ui = *reinterpret_cast<unsigned int *>(cpp);
+
+            if (ui < 0)
+                py = PyLong_FromUnsignedLong((unsigned long)ui);
+            else
+                py = SIPLong_FromLong(ui);
+
+            break;
+        }
 
     case QMetaType::Double:
         py = PyFloat_FromDouble(*reinterpret_cast<double *>(cpp));
